package com.kimchiapps.peeble;

import android.app.Notification;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;

import androidx.core.app.NotificationCompat;
import androidx.core.app.NotificationManagerCompat;
import androidx.core.app.Person;
import androidx.fragment.app.Fragment;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.google.android.gms.auth.api.signin.GoogleSignIn;
import com.google.android.gms.auth.api.signin.GoogleSignInAccount;
import com.google.firebase.auth.AuthCredential;
import com.google.firebase.auth.FirebaseAuth;

import co.intentservice.chatui.ChatView;
import co.intentservice.chatui.models.ChatMessage;

import com.google.firebase.auth.GoogleAuthProvider;
import com.google.firebase.database.ChildEventListener;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;

import java.util.Date;

/**
 * A simple {@link Fragment} subclass.
 * Activities that contain this fragment must implement the
 * {@link Chat.OnFragmentInteractionListener} interface
 * to handle interaction events.
 * Use the {@link Chat#newInstance} factory method to
 * create an instance of this fragment.
 */
public class Chat extends Fragment {

    private static Context context; // getApplicationContext();

    private GoogleSignInAccount account;

    private OnFragmentInteractionListener mListener;

    private ChatView chatView; // Chat UI

    private static boolean sentMessage;

    public Chat() {
        // Required empty public constructor
    }

    /**
     * Use this factory method to create a new instance of
     * this fragment using the provided parameters.
     *
     * @return A new instance of fragment Chat.
     */

    static Chat newInstance() {
        Chat fragment = new Chat();
        Bundle args = new Bundle();
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        context = this.getContext(); // getApplicationContext();

        account = GoogleSignIn.getLastSignedInAccount(context);

        // Initialize FirebaseAuth
        // Firebase instance variables
        FirebaseAuth mFirebaseAuth = FirebaseAuth.getInstance();
        AuthCredential credential = GoogleAuthProvider.getCredential(account.getIdToken(), null);
        FirebaseAuth.getInstance().signInWithCredential(credential);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_chat, container, false);
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        sentMessage = false;

        chatView = (ChatView) view.findViewById(R.id.chat_view);

        chatView.setOnSentMessageListener(new ChatView.OnSentMessageListener(){
            @Override
            public boolean sendMessage(ChatMessage chatMessage){
                // perform actual message sending

                Message message = new Message(chatMessage, account);

                // Read the input field and push a new instance
                // of message to the Firebase database
                FirebaseDatabase.getInstance()
                        .getReference()
                        .child("messages")
                        .push()
                        .setValue(message);

                sentMessage = true;

                return true;
            }
        });

        DatabaseReference messages = FirebaseDatabase.getInstance().getReference("messages");

        messages.addChildEventListener(new ChildEventListener() {

            @Override
            public void onChildAdded(DataSnapshot dataSnapshot, String prevChildKey) { // New message added to database
                Message newMessage = dataSnapshot.getValue(Message.class);

                if (newMessage.getSender().equals(account.getEmail()) && !sentMessage) { // If from user
                    chatView.addMessage(new ChatMessage(newMessage.getMessage(), newMessage.getTimestamp(), ChatMessage.Type.SENT));
                }
                else if (!newMessage.getSender().equals(account.getEmail())) { // If from another user from another device
                    chatView.addMessage(new ChatMessage(newMessage.getMessage(), newMessage.getTimestamp(), ChatMessage.Type.RECEIVED, newMessage.getSenderName()));

                    Chat chat;
                    try {
                        chat = (Chat) getFragmentManager().findFragmentByTag("chatFragment");
                    }
                    catch (NullPointerException e) {
                        chat = null;
                    }

                    if(getContext() != null && getContext() != context)
                        context = getContext();

                    // Create notification if chatView is not visible
                    if(newMessage.getTimestamp() >= new Date().getTime() - 1000 && !MainScreen.isAppOnForeground(context) || chat == null || !chat.isVisible()) { // Give 1 sec buffer time
                        Person user = new Person.Builder()
                                .setName(newMessage.getSenderName())
                                .build();

                        NotificationCompat.MessagingStyle.Message messageNotification =
                                new NotificationCompat.MessagingStyle.Message(newMessage.getMessage(),
                                        newMessage.getTimestamp(),
                                        user);

                        Intent notificationIntent = new Intent(context, MainScreen.class);

                        PendingIntent pendingIntent = PendingIntent.getActivity(context, 0, notificationIntent, 0);

                        Notification notification = new NotificationCompat.Builder(context, "chatNotification")
                                .setSmallIcon(R.drawable.pblicon)
                                .setStyle(new NotificationCompat.MessagingStyle(user)
                                        .addMessage(messageNotification))
                                .setContentIntent(pendingIntent)
                                .setAutoCancel(true)
                                .build();

                        NotificationManagerCompat notificationManager = NotificationManagerCompat.from(context);

                        notificationManager.notify(0, notification);
                    }
                }
            }

            @Override
            public void onChildChanged(DataSnapshot dataSnapshot, String prevChildKey) {}

            @Override
            public void onChildRemoved(DataSnapshot dataSnapshot) {}

            @Override
            public void onChildMoved(DataSnapshot dataSnapshot, String prevChildKey) {}

            @Override
            public void onCancelled(DatabaseError databaseError) {}
        });
    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof OnFragmentInteractionListener) {
            mListener = (OnFragmentInteractionListener) context;
        } else {
            throw new RuntimeException(context.toString()
                    + " must implement OnFragmentInteractionListener");
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    /**
     * This interface must be implemented by activities that contain this
     * fragment to allow an interaction in this fragment to be communicated
     * to the activity and potentially other fragments contained in that
     * activity.
     * <p>
     * See the Android Training lesson <a href=
     * "http://developer.android.com/training/basics/fragments/communicating.html"
     * >Communicating with Other Fragments</a> for more information.
     */
    public interface OnFragmentInteractionListener {
        void onFragmentInteraction(Uri uri);
    }
}